/*
** Module   : initterm_win.c
** Abstract : implementation for p2j.dll native library initialization and termination routine.
**            This is used by DLL startup and unloading code.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 EVL 20130430 Created initial version to implement Windows specifics for DLL init/term code.
*/

#include <stdio.h> // Remove this after printf() debugging no longer needs
#include <windows.h>

// C Runtime initialization prototype
BOOL WINAPI _CRT_INIT(HINSTANCE handle, DWORD reason, LPVOID reserved); 

/**
* The DLL entry point.
* 
* @param  handle 
*         Handle of the DLL module to load/unload
* @param  reason 
*         The flag field indicating the current operation and why the entry point has been called
* @param  reserved 
*         The pointer to the reserver parameter. Depends on the operation code flags:
*         - DLL_PROCESS_ATTACH: NULL for dynamic loads, non-NULL for static loads
*         - DLL_PROCESS_DETACH: NULL if FreeLibrary has been called or DLL load was failed,
*                               non-NULL if process is terminating
* 
* @return TRUE in case of success, FALSE otherwise.
*/
BOOL WINAPI DllEntryPoint(HINSTANCE handle, DWORD reason, LPVOID reserved)
{
   // The part of initialization
   if (reason == DLL_PROCESS_ATTACH || reason == DLL_THREAD_ATTACH)
   {
      // Still not possible to access C runtime functions
      if (!_CRT_INIT(handle, reason, reserved))
      {
         return (FALSE);
      }

      // We can use C runtime from this point
      if (reason == DLL_PROCESS_ATTACH)
      {
         // The process is loading DLL
         // The process specific initialization code should be placed here
      }
      else
      {
         // Another thread started and attached to the DLL
         // Thread-specific initialization here
      }
   }

   // The part of detach and termination
   if (reason == DLL_PROCESS_DETACH || reason == DLL_THREAD_DETACH)
   {
      // The final place to call C runtime
      if (reason == DLL_PROCESS_DETACH)
      {
         // The process is unloading DLL
      }
      else
      {
         // Another thread finishes the working with DLL, cleanup requested
      }

      // Now we are loosing the ability to call C runtime functions
      if (!_CRT_INIT(handle, reason, reserved))
      {
         return (FALSE);
      }
   }
   
   return (TRUE);
}
